package okay.collections;

import okay.OkayRuntimeException;

/**
 * Is used to iterate entries through dictionary.
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class DictionaryKeyIterator {

    private final DictionaryNode firstNode;
    private DictionaryNode currentNode;
    private boolean currentNodeIsNotFirst = false;

    DictionaryKeyIterator(DictionaryNode value) {
        this.firstNode = value;
        this.reset();
    }

    /**
     *
     * @return next key
     */
    public String getNextKey() {
        String key;
        if (firstNode == null) {
            throw new OkayRuntimeException("Dictionary is empty. There is no key.");
        }
        if ((currentNode == firstNode) && (!this.currentNodeIsNotFirst)) {
            key = currentNode.getKey();
            this.currentNodeIsNotFirst = true;
        } else {
            if (!hasNext()) {
                throw new OkayRuntimeException("There is no next key.");
            }
            this.currentNode = this.currentNode.getNext();
            key = currentNode.getKey();
        }
        return key;
    }

    /**
     *
     * @return true if there is a next key, otherwise false
     */
    public boolean hasNext() {
        if (this.firstNode == null) {
            return false;
        }
        if ((firstNode.getNext() == null) && (!currentNodeIsNotFirst)) {
            return true;
        }
        if ((firstNode.getNext() == null) && (currentNodeIsNotFirst)) {
            return false;
        }
        return this.currentNode.getNext() != null;
    }

    /**
     * Moves the current key to the first.
     */
    public void reset() {
        this.currentNode = this.firstNode;
        currentNodeIsNotFirst = false;
    }
}
