package okay.collections;

import okay.OkayRuntimeException;

/**
 * Represents Queue- linear data structure.
 *
 * @author Robert Vokáč robertvokac1@gmail.comt Vokáč robertvokac1@gmail.com
 * @param <T> object
 */
public class Queue<T> {

    private int total;

    private Node first;
    private Node last;

    private class Node {

        private T element;
        private Node next;
    }

    /**
     *
     * @return size of this queue
     */
    public int getCountOfItems() {
        return this.total;
    }

    /**
     *
     * @return true if this queue is empty, otherwise false
     */
    public boolean isEmpty() {
        return this.getCountOfItems() == 0;
    }

    /**
     * Enqueues new element.
     *
     * @param element
     * @return new element
     */
    public Queue<T> enqueue(T element) {
        Node current = last;
        last = new Node();
        last.element = element;

        if (total++ == 0) {
            first = last;
        } else {
            current.next = last;
        }

        return this;
    }

    /**
     * Dequeues element.
     *
     * @return removed element
     */
    public T dequeue() {
        if (total == 0) {
            throw new OkayRuntimeException("No such element");
        }
        T ele = first.element;
        first = first.next;
        if (--total == 0) {
            last = null;
        }
        return ele;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        Node tmp = first;
        while (tmp != null) {
            sb.append(tmp.element).append(", ");
            tmp = tmp.next;
        }
        return sb.toString();
    }
}
