package okay.collections;

import java.util.Iterator;
import java.util.NoSuchElementException;
import okay.OkayRuntimeException;

/**
 * Represents Stack- linear data structure.
 *
 * @author Robert Vokáč robertvokac1@gmail.comt Vokáč robertvokac1@gmail.com
 * @param <T> object
 */
public class Stack<T> implements Iterable<T> {

    private int total = 0;

    private Node first;

    @Override
    public Iterator<T> iterator() {
        return new StackIterator();
    }

    private class Node {

        private T element;
        private Node next;
    }

    private class StackIterator implements Iterator<T> {

        @Override
        public boolean hasNext() {
            return !isEmpty();
        }

        @Override
        public T next() {
            if (!hasNext()) {
                throw new NoSuchElementException();
            }
            return pop();
        }
    }

    /**
     *
     * @return size of this stack
     */
    public int getCountOfItems() {
        return this.total;
    }

    /**
     *
     * @return true if this stack is empty, otherwise false
     */
    public boolean isEmpty() {
        return this.getCountOfItems() == 0;
    }

    /**
     * Pushes new element.
     *
     * @param element
     * @return pushed element
     */
    public Stack<T> push(T element) {
        Node current = first;
        first = new Node();
        first.element = element;
        first.next = current;
        total++;
        return this;
    }

    /**
     * Pops element.
     *
     * @return popped element
     */
    public T pop() {
        if (first == null) {
            throw new OkayRuntimeException("No such element");
        }
        T element = first.element;
        first = first.next;
        total--;
        return element;
    }

    /**
     *
     * @return the top element of this stack without removing this element
     */
    public T peek() {
        if (first == null) {
            throw new OkayRuntimeException("No such element");
        }
        return first.element;
    }
/**
 * Deletes all items of this stack.
 */
    public void clear() {
        total = 0;
        first = null;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        Node tmp = first;
        while (tmp != null) {
            sb.append(tmp.element).append(", ");
            tmp = tmp.next;
        }
        return sb.toString();
    }

}
