package okay.datetime;

import java.time.format.DateTimeFormatter;

/**
 * This class stores date time without time zone information.
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public final class LocalDateTime extends DateTime {

    /**
     * Constructor
     *
     * Creates new LocalDateTime from String.
     *
     * @param dateTimeInString
     */
    protected LocalDateTime(String dateTimeInString) {
        super(dateTimeInString);
    }

    /**
     * Constructor
     *
     * Creates new LocalDateTime with these parameters.
     *
     * @param year
     * @param month
     * @param day
     * @param hour24Format
     * @param minute
     * @param second
     * @param millisecond
     */
    public LocalDateTime(int year, int month, int day, int hour24Format, int minute, int second, int millisecond) {
        super(year, month, day, hour24Format, minute, second, millisecond);
    }

    /**
     * Constructor
     *
     * Creates new LocalDateTime from UniversalDateTime.
     *
     * @param universalDateTime
     */
    public LocalDateTime(UniversalDateTime universalDateTime) {
        super(universalDateTime);
    }

    /**
     * Constructor
     *
     * Creates new LocalDateTime from ZonedDateTime.
     *
     * @param zonedDateTime
     */
    public LocalDateTime(ZonedDateTime zonedDateTime) {
        super(zonedDateTime);
    }

    /**
     *
     *
     * @return a UniversalDateTime instance from this object
     */
    public UniversalDateTime toUniversalDateTime() {
        return new UniversalDateTime(this);
    }

    /**
     *
     *
     * @param timeZone time zone of new created ZonedDateTime
     * @return a ZonedDateTime instance from this object
     */
    public ZonedDateTime toZonedDateTime(okay.datetime.TimeZone timeZone) {
        return new ZonedDateTime(this, timeZone);
    }

    /**
     *
     *
     * @return a java.time.LocalDateTime instance from this object
     */
    java.time.LocalDateTime toJavaLocalDateTime() {
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss:SSS");
        String stringRepresentationOfThisObject = this.toString();
        return java.time.LocalDateTime.parse(stringRepresentationOfThisObject, dateTimeFormatter);
    }

}
