package okay.datetime;

import okay.OkayRuntimeException;

/**
 * Represents Time without date and time zone information.
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public final class LocalTime {

    private static final String COLON = ":";

    private final int hour;
    private final int minute;
    private final int second;
    private final int millisecond;

    /**
     * Constructor
     *
     * @param hour
     * @param minute
     * @param second
     * @param millisecond
     * @exception OkayRuntimeException if parameters are invalid.
     */
    public LocalTime(int hour, int minute, int second, int millisecond) {
        TimeUnitsValidator.checkInputValuesForTimeAndIfThereIsAnInvalidOneThrowException(hour, minute, second, millisecond);
        this.hour = hour;
        this.minute = minute;
        this.second = second;
        this.millisecond = millisecond;
    }

    /**
     *
     * @return hour of this time.
     */
    public int getHour() {
        return hour;
    }

    /**
     *
     * @return minute of this time.
     */
    public int getMinute() {
        return minute;
    }

    /**
     *
     * @return second of this time.
     */
    public int getSecond() {
        return second;
    }

    /**
     *
     * @return millisecond of this time.
     */
    public int getMillisecond() {
        return millisecond;
    }

    @Override
    public String toString() {
        StringBuilder stringBuilder = new StringBuilder();

        if (hour < 10) {
            stringBuilder.append("0");
        }
        stringBuilder.append(this.getHour());
        stringBuilder.append(COLON);
        if (minute < 10) {
            stringBuilder.append("0");
        }
        stringBuilder.append(this.getMinute());
        stringBuilder.append(COLON);
        if (second < 10) {
            stringBuilder.append("0");
        }
        stringBuilder.append(this.getSecond());
        stringBuilder.append(COLON);
        if (millisecond < 10) {
            stringBuilder.append("00");
        } else if (millisecond < 100) {
            stringBuilder.append("0");
        }
        stringBuilder.append(this.getMillisecond());
        return stringBuilder.toString();
    }
}
