package okay.datetime;

import java.text.DateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * Is used to get current universal date and time.
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
class TimeSource {

    private static final okay.datetime.TimeZone universalTimeZone = new okay.datetime.TimeZone("GMT");

    private static final DateFormat yearFormat = new DateTime.SimpleDateTimeFormatByTimeZone(universalTimeZone, "yyyy");
    private static final DateFormat monthFormat = new DateTime.SimpleDateTimeFormatByTimeZone(universalTimeZone, "MM");
    private static final DateFormat dayFormat = new DateTime.SimpleDateTimeFormatByTimeZone(universalTimeZone, "dd");
    private static final DateFormat hourFormat = new DateTime.SimpleDateTimeFormatByTimeZone(universalTimeZone, "HH");
    private static final DateFormat minuteFormat = new DateTime.SimpleDateTimeFormatByTimeZone(universalTimeZone, "mm");
    private static final DateFormat secondFormat = new DateTime.SimpleDateTimeFormatByTimeZone(universalTimeZone, "ss");
    private static final DateFormat millisecondFormat = new DateTime.SimpleDateTimeFormatByTimeZone(universalTimeZone, "S");

    /**
     * Represents source for current universal date time.
     */
    private Calendar calendar;
    private final UniversalDateTime universalDateTime;

    /**
     * Constructor
     *
     * Every time, you want know the current time, you have to create new
     * instance of TimeSource class.
     */
    private TimeSource() {
        this.universalDateTime = this.buildCurrentUniversalDateTime();
    }

    /**
     *
     * @return current universal date time
     */
    public static UniversalDateTime getCurrentUniversalDateTimeInstance() {
        TimeSource timeSource = new TimeSource();
        return timeSource.getCurrentUniversalDateTime();
    }

    /**
     *
     * @return Date instance for current universal date and time
     */
    private UniversalDateTime buildCurrentUniversalDateTime() {
        Date currentDate = getCurrentDate();
        int year = Integer.parseInt(yearFormat.format(currentDate));
        int month = Integer.parseInt(monthFormat.format(currentDate));
        int day = Integer.parseInt(dayFormat.format(currentDate));

        int hour = Integer.parseInt(hourFormat.format(currentDate));
        int minute = Integer.parseInt(minuteFormat.format(currentDate));
        int second = Integer.parseInt(secondFormat.format(currentDate));
        int millisecond = Integer.parseInt(millisecondFormat.format(currentDate));
        return new UniversalDateTime(year, month, day, hour, minute, second, millisecond);
    }

    /**
     *
     * @return a new instance of Date class for current universal date and time
     */
    private Date getCurrentDate() {
        calendar = Calendar.getInstance();
        Date currentDate = calendar.getTime();
        calendar = null;
        return currentDate;
    }

    /**
     *
     * @return current universal date time
     */
    UniversalDateTime getCurrentUniversalDateTime() {
        return this.universalDateTime;
    }
}
