package okay.json;

import java.util.ArrayList;
import okay.OkayRuntimeException;

/**
 * Is used to create json arrays from Strings.
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
class JsonArrayParser extends JsonParser {

    /**
     * Constructor
     *
     * Not meant to be instantiated.
     */
    private JsonArrayParser() {
        //Not meant to be instantiated.
    }

    /**
     * Parses String to empty json object.
     *
     * @param jsonObject
     * @param stringRepresentationOfJsonObject
     * @return json object from parsed String
     */
    static JsonArray parseStringToJsonArray(JsonArray jsonArray, String stringRepresentationOfJsonArray) {
        if (!jsonArray.isEmpty()) {
            throw new OkayRuntimeException("I can't parse. The json array is not empty.");
        }
        String trimmedStringRepresentationOfJsonArray = stringRepresentationOfJsonArray.trim();
        if (!isStringJsonArray(trimmedStringRepresentationOfJsonArray)) {
            throw new OkayRuntimeException("I can't parse. The trimmed String does not start with [ or does not end with ]");
        }
        String collectionsOfValuesOfTheJsonArrayString = deleteTheCharAtTheStartAndTheEnd(trimmedStringRepresentationOfJsonArray);
        if ("".equals(collectionsOfValuesOfTheJsonArrayString)) {
            return jsonArray;
        }
        fillJsonArrayWithParsedValues(jsonArray, collectionsOfValuesOfTheJsonArrayString);
        return jsonArray;
    }

    private static boolean isStringJsonArray(String trimmedStringRepresentationOfJsonArray) {
        return (getFirstCharOfTheString(trimmedStringRepresentationOfJsonArray) == JsonSpecialCharSequences.getArrayLeft()) && (getLastCharOfTheString(trimmedStringRepresentationOfJsonArray) == JsonSpecialCharSequences.getArrayRight());
    }

    private static void fillJsonArrayWithParsedValues(JsonArray jsonArray, String collectionsOfValuesOfTheJsonArrayString) {

        ArrayList<Integer> listOfCommas = getListOfIndexesOfTheStringWhereCharIsCommaAndNestingIsZero(collectionsOfValuesOfTheJsonArrayString);

        int beginIndex;
        int indexOfComma = 0;
        for (int i = 0; i < listOfCommas.size(); i++) {
            beginIndex = ++indexOfComma;
            indexOfComma = listOfCommas.get(i);
            if (i == 0) {
                beginIndex = 0;
            }
            String fraction = collectionsOfValuesOfTheJsonArrayString.substring(beginIndex, indexOfComma);
            Object parsedObject = JsonParser.parseStringToValue(fraction);
            jsonArray.add(parsedObject);
        }

    }

}
