package okay.json;

/**
 * Used to create String representation of a json array.
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
class JsonArrayPrinter {

    /**
     * Constructor
     *
     * Not meant to be instantiated.
     */
    private JsonArrayPrinter() {
        //Not meant to be instantiated.
    }

    /**
     *
     * @return representation of the json array as pretty String
     */
    static String toMinimalString(JsonArray jsonArray) {
        return toString(jsonArray, false);
    }

    /**
     *
     * @return representation of the json array as pretty String
     */
    static String toPrettyString(JsonArray jsonArray) {
        return toString(jsonArray, true);

    }

    /**
     *
     * @return representation of the json array as String
     */
    static String toString(JsonArray jsonArray, boolean printAsPrettyVersion) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(JsonSpecialCharSequences.getArrayLeft());
        if (printAsPrettyVersion) {
            stringBuilder.append(JsonSpecialCharSequences.getLineBreak());
        }
        int i = 0;

        String value;
        while (i != jsonArray.getCountOfItems()) {
            if (printAsPrettyVersion) {
                value = jsonArray.getJsonValue(i).toPrettyString();
            } else {
                value = jsonArray.getJsonValue(i).toMinimalString();
            }
            if (printAsPrettyVersion) {
                if (!((jsonArray.getJsonValue(i).isArray()) || (jsonArray.getJsonValue(i).isObject()))) {
                    stringBuilder.append(JsonSpecialCharSequences.getTab());
                } else {
                    value = JsonPrinter.addTabOnEveryLineStart(value);
                }
            }

            stringBuilder.append(value);
            if ((i + 1) != jsonArray.getCountOfItems()) {
                stringBuilder.append(",");
            }
            if (printAsPrettyVersion) {
                stringBuilder.append("\n");
            }
            i++;
        }
        stringBuilder.append(JsonSpecialCharSequences.getArrayRight());

        return stringBuilder.toString();

    }

}
