package okay.json;

import okay.collections.DictionaryKeyIterator;

/**
 * Used to create String representation of a json object.
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
class JsonObjectPrinter extends JsonPrinter {

    /**
     * Constructor
     *
     * Not meant to be instantiated.
     */
    private JsonObjectPrinter() {
        //Not meant to be instantiated.
    }

    /**
     * @param jsonObject
     *
     * @return String representation of this json object in minimal version
     */
    static String toMinimalString(JsonObject jsonObject) {
        return toString(jsonObject, false);
    }

    /**
     * @param jsonObject
     *
     * @return String representation of this json object in pretty print version
     */
    static String toPrettyString(JsonObject jsonObject) {
        return toString(jsonObject, true);
    }

    /**
     * @param jsonObject
     * @param printAsPrettyVersion
     *
     * @return String representation of this json object
     */
    static String toString(JsonObject jsonObject, boolean printAsPrettyVersion) {
        DictionaryKeyIterator keyIterator = jsonObject.getKeyIterator();
        String value;
        String key;

        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(JsonSpecialCharSequences.getObjectLeft());
        if (printAsPrettyVersion) {
            stringBuilder.append(JsonSpecialCharSequences.getLineBreak());
        }

        while (keyIterator.hasNext()) {
            key = keyIterator.getNextKey();
            if (printAsPrettyVersion) {
                value = jsonObject.getJsonValue(key).toPrettyString();
            } else {
                value = jsonObject.getJsonValue(key).toMinimalString();
            }

            if (printAsPrettyVersion) {
                stringBuilder.append(JsonSpecialCharSequences.getTab());
            }
            stringBuilder.append(JsonSpecialCharSequences.getApostrophe());
            stringBuilder.append(key);
            stringBuilder.append(JsonSpecialCharSequences.getApostrophe());
            stringBuilder.append(JsonSpecialCharSequences.getColon());

            if (printAsPrettyVersion && ((jsonObject.getJsonValue(key).isArray()) || (jsonObject.getJsonValue(key).isObject()))) {
                value = addTabOnEveryLineStartWithoutFirst(value);
            }

            stringBuilder.append(value);
            if (keyIterator.hasNext()) {
                stringBuilder.append(",");
            }
            if (printAsPrettyVersion) {
                stringBuilder.append(JsonSpecialCharSequences.getLineBreak());
            }
        }
        stringBuilder.append(JsonSpecialCharSequences.getObjectRight());

        return stringBuilder.toString();
    }
}
