package okay.pseudorandom;

import okay.OkayRuntimeException;
import okay.datetime.DateTime;
import okay.datetime.UniversalDateTime;

/**
 * Used to generate pseudo random values.
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class PseudoRandomGenerator {

    private final Seed magicSeed;
    private final Seed[] seed = new Seed[2];
    private static final PseudoRandomGenerator pseudoRandomGenerator;

    /**
     * Constructor
     *
     * @param magicNumber
     * @param dateTime
     */
    public PseudoRandomGenerator(long magicNumber, DateTime dateTime) {
        this.magicSeed = new Seed(magicNumber);
        long year = dateTime.getYear();
        long month = dateTime.getMonth();
        long day = dateTime.getDay();
        long hour = dateTime.getHour();
        long minute = dateTime.getMinute();
        long second = dateTime.getSecond();
        long millisecond = dateTime.getMillisecond();
        this.seed[0] = new Seed(1000 * (minute + day) + millisecond);
        this.seed[1] = new Seed(hour * 100000000 + year * 10000 + second * 100 + month);

    }

    static {
        UniversalDateTime currentUniversalDateTime = UniversalDateTime.getCurrentUniversalDateTime();
        int currentDay = currentUniversalDateTime.getMonth();
        int currentMillisecond = currentUniversalDateTime.getMillisecond();
        long magicNumber = (long) currentDay * 1000 + (long) currentMillisecond;
        pseudoRandomGenerator = new PseudoRandomGenerator(magicNumber, currentUniversalDateTime);
    }

    /**
     * @return static instance
     */
    public static PseudoRandomGenerator getInstance() {
        return pseudoRandomGenerator;
    }

    /**
     *
     * @param from
     * @param to
     * @return pseudo random long
     */
    public long getLong(long from, long to) {
        if (from > to) {
            throw new IllegalArgumentException("from is greater than to.");
        }

        long[] numberFractions = new long[2];

        for (int i = 0; i <= 1; i++) {
            this.seed[0].jump();
            this.seed[1].jump();

            if (((magicSeed.getNextNumber()) % 2) == 0) {
                this.seed[0].jump();
            }
            if (((magicSeed.getNextNumber()) % 2) == 1) {
                this.seed[1].jump();
            }
            long number1 = seed[0].getNextNumber();
            long number2 = seed[1].getNextNumber();
            if ((magicSeed.getNextNumber() % 2) == 1) {
                number2 = number2 * (-1);
            }
            numberFractions[i] = (Math.abs(number1 + number2)) % 1000000000;
        }
        long number = (numberFractions[0] * 1000000000) + numberFractions[1];

        number = (number % (to - from + 1)) + from;

        return number;
    }

    /**
     *
     * @param from
     * @param to
     * @return pseudo random int
     */
    public int getInt(int from, int to) {
        if (to > Integer.MAX_VALUE) {
            throw new OkayRuntimeException("The parameter \"to\" is too big.");
        }
        return (int) getLong(from, to);
    }

    /**
     *
     * @return pseudo random boolean
     */
    public boolean getBoolean() {
        int value = getInt(0, 1);
        return value == 1;
    }
}
