package okay.simplicity;

import okay.OkayRuntimeException;
import okay.pseudorandom.PseudoRandomGenerator;

/**
 * Represents one of 16 colours, or there is no colour.
 *
 * @author Robert Vokáč robertvokac1@gmail.com
 */
public enum EnumColour {

    /**
     * No colour is selected.
     */
    NOCOLOUR,
    /**
     * Light Green
     */
    LIGHTGREEN,
    /**
     * Red
     */
    RED,
    /**
     * Dark Blue
     */
    DARKBLUE,
    /**
     * Yellow
     */
    YELLOW,
    /**
     * Light Blue
     */
    LIGHTBLUE,
    /**
     * Purple
     */
    PURPLE,
    /**
     * Brown
     */
    BROWN,
    /**
     * Pink
     */
    PINK,
    /**
     * Green
     */
    GREEN,
    /**
     * Gold
     */
    GOLD,
    /**
     * Orange
     */
    ORANGE,
    /**
     * White
     */
    WHITE,
    /**
     * Grey
     */
    GREY,
    /**
     * Black
     */
    BLACK,
    /**
     * Blue
     */
    BLUE,
    /**
     * OliveGreen
     */
    OLIVEGREEN;

    /**
     *
     * @return
     * <p>
     * ordinal number of enum colour</p>
     * <p>
     * Converts EnumColour to int.</p>
     */
    public int getNumber() {
        return this.ordinal();
    }

    /**
     *
     * @param number
     * @return
     */
    public static EnumColour convertNumberToEnumColour(int number) {//NOSONAR
        if (!isNumberRepresentationValid(number)) {
            throw new OkayRuntimeException("Number you want co convert to EnumColour must be at least 0 and at most 16.");
        }
        switch (number) {
            case 0:
                return EnumColour.NOCOLOUR;
            case 1:
                return EnumColour.LIGHTGREEN;
            case 2:
                return EnumColour.RED;
            case 3:
                return EnumColour.DARKBLUE;
            case 4:
                return EnumColour.YELLOW;
            case 5:
                return EnumColour.LIGHTBLUE;
            case 6:
                return EnumColour.PURPLE;
            case 7:
                return EnumColour.BROWN;
            case 8:
                return EnumColour.PINK;
            case 9:
                return EnumColour.GREEN;
            case 10:
                return EnumColour.GOLD;
            case 11:
                return EnumColour.ORANGE;
            case 12:
                return EnumColour.WHITE;
            case 13:
                return EnumColour.GREY;
            case 14:
                return EnumColour.BLACK;
            case 15:
                return EnumColour.BLUE;
            case 16:
                return EnumColour.OLIVEGREEN;
            default:
                return null;

        }
    }

    private static boolean isNumberRepresentationValid(int number) {
        return (number >= 0) && (number <= 16);
    }

    /**
     *
     * @return random EnumColour
     */
    public static EnumColour getRandom() {
        int i = PseudoRandomGenerator.getInstance().getInt(1, 16);
        return EnumColour.convertNumberToEnumColour(i);
    }
}
