package okay.simplicity;

import javafx.geometry.Rectangle2D;

/**
 * Represents screen.
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public final class Screen {

    private static double dpmm;
    private static double dpi;
    private static double zoom = 100;

    /**
     * Constructor
     *
     * Not meant to be instantiated.
     */
    private Screen() {
        //Not meant to be instantiated.
    }

    /**
     * Initialises screen.
     */
    static void initScreen() {
        setDpi();
        updateDpmm();
    }

    /**
     * Sets dots per inch.
     */
    private static void setDpi() {
        javafx.stage.Screen primaryScreen = javafx.stage.Screen.getPrimary();
        dpi = primaryScreen.getDpi();
    }

    /**
     * @return the resolution (dots per inch) of screen.
     */
    public static double getDpi() {
        return dpi;
    }

    /**
     * Sets the length of one millimetre long line in dots (dots per
     * millimetre.)
     */
    private static void updateDpmm() {
        double currentDpi = getDpi();
        if (currentDpi > 0) {
            dpmm = currentDpi / 25.4;
        } else {
            dpmm = 6;
        }
        dpmm = dpmm * (getZoom() / 100);
    }

    /**
     * @return the length of one millimetre long line in dots (dots per
     * millimetre).
     */
    public static double getDpmm() {
        return dpmm;
    }

    /**
     *
     * @return zoom
     */
    public static double getZoom() {
        return zoom;
    }

    /**
     *
     * @param zoom 100% is 100, 100% is default
     */
    public static void setZoom(double newZoom) {
        zoom = newZoom;
        updateDpmm();
    }

    private static Rectangle2D getBounds() {
        return javafx.stage.Screen.getPrimary().getBounds();
    }

    /**
     *
     * @return screen width
     */
    public static int getWidth() {
        return (int) getBounds().getWidth();
    }

    /**
     *
     * @return screen height
     */
    public static int getHeight() {
        return (int) getBounds().getHeight();
    }

    private static Rectangle2D getVisualBounds() {
        return javafx.stage.Screen.getPrimary().getVisualBounds();
    }

    /**
     *
     * @return screen visual width
     */
    public static int getVisualWidth() {
        return (int) getVisualBounds().getWidth();
    }

    /**
     *
     * @return screen visual height
     */
    public static int getVisualHeight() {
        return (int) getVisualBounds().getHeight();
    }
}
