package okay.simplicity;

import javafx.application.Platform;
import okay.simplicity.window.WindowColourSkin;

/**
 * Represents Simplicity package main class.
 *
 * @author Robert Vokáč robertvokac1@gmail.com
 */
public final class Simplicity {

    private static SimplicityRunnerI simplicityRunner;
    private static WindowColourSkin defaultWindowColourSkin;

    /**
     * Constructor
     *
     * Not meant to be instantiated.
     */
    private Simplicity() {
        //Not meant to be instantiated.
        Platform.setImplicitExit(false);
    }
    /**
     * This method must be called before using Simplicity.
     *
     * @param runner This object implements runObjectI interface and its method
     * run contains some logic.
     * @param defaultWindowColourSkin
     */
    public static void startSimplicity(SimplicityRunnerI runner, WindowColourSkin defaultWindowColourSkin) {
        Simplicity.setDefaultWindowColourSkin(defaultWindowColourSkin);
        simplicityRunner = runner;
        JavaFXApplication.startJavaFXApplication();
    }

    /**
     * Stops Simplicity.
     */
    public static void stopSimplicity() {
        Platform.exit();
    }

    /**
     *
     * @return Simplicity runner
     */
    public static SimplicityRunnerI getSimplicityRunner() {
        return simplicityRunner;
    }

    /**
     * Returns count of dots per one millimeter on the screen.
     *
     * @return count of dots per one millimeter on the screen
     */
    public static double getDpmm() {
        return Screen.getDpmm();
    }

    /**
     *
     * @return default window colour skin
     */
    public static WindowColourSkin getDefaultWindowColourSkin() {
        return Simplicity.defaultWindowColourSkin;
    }

    /**
     *
     * @param windowColourSkin
     */
    public static void setDefaultWindowColourSkin(WindowColourSkin windowColourSkin) {
        Simplicity.defaultWindowColourSkin = windowColourSkin;
    }

    public static String getInformation() {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("\ndefaultWindowColourSkin: ").append(Simplicity.getDefaultWindowColourSkin());
        stringBuilder.append("\ndpmm: ").append(Simplicity.getDpmm());
        stringBuilder.append("\nzoom: ").append(Screen.getZoom());
        stringBuilder.append("\ndpi: ").append(Screen.getDpi());
        stringBuilder.append("\nheight: ").append(Screen.getHeight());
        stringBuilder.append("\nwidth: ").append(Screen.getWidth());
        stringBuilder.append("\nvisual height: ").append(Screen.getVisualHeight());
        stringBuilder.append("\nvisual width: ").append(Screen.getVisualWidth());
        return stringBuilder.toString();
    }
}
