package okay.simplicity.window;

import okay.simplicity.Simplicity;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import static javafx.scene.layout.HBox.setHgrow;
import javafx.scene.layout.Priority;
import javafx.scene.layout.StackPane;
import javafx.scene.text.Font;
import javafx.scene.text.Text;

/**
 * Represents the part of window, where is the window title placed and is used
 * to move window
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class MoveableArea extends StackPane {

    private final Text titleText = new Text("");
    private final Window window;
    private boolean moveable;

    /**
     * Constructor
     *
     * @param window
     */
    public MoveableArea(Window window) {
        this.window = window;
        getChildren().add(titleText);
        setStyle("-fx-background-color: rgb(" + Simplicity.getDefaultWindowColourSkin().getRGBStringForCSS(ColourVariant.DARK) + ");");
        setAlignment(Pos.CENTER_LEFT);

        setHgrow(this, Priority.ALWAYS);

        titleText.setFont(Font.font(Simplicity.getDpmm() * 3));
        titleText.setFill(Simplicity.getDefaultWindowColourSkin().getColourForText(ColourVariant.DARK));
        titleText.setScaleX(1.2);
        setPadding(new Insets(0, 0, 0, Simplicity.getDpmm()));
        this.setCanBeMoved(true);
    }

    /**
     *
     * @param value
     */
    public void setCanBeMoved(boolean value) {
        moveable = value;
        if (value) {
            setOnMousePressed(window::handleMovingWindowStarted);
            setOnMouseDragged(window::handleMovingWindowEnded);
        } else {
            setOnMousePressed(null);
            setOnMouseDragged(null);
        }

    }

    /**
     *
     * @return true, if this object is able to move window, otherwise false
     */
    public boolean canBeMoved() {
        return this.moveable;

    }

    /**
     *
     * @param title
     */
    public void setWindowTitle(String title) {
        titleText.setText("  " + title);
    }

    /**
     *
     * @return window title
     */
    public String getWindowTitle() {
        return this.titleText.getText();
    }
}
