package okay.simplicity.window.controls;

import okay.simplicity.Simplicity;
import okay.simplicity.window.ColourVariant;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.HBox;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Line;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class SCheckBox extends HBox {

    private StackPane checkBoxStackPane = new StackPane();
    private SLabel checkBoxSLabel = new SLabel();
    private boolean selected = false;
    private boolean disabled = false;
    Line line1 = new Line(0, 0, 3 * Simplicity.getDpmm(), 3 * Simplicity.getDpmm());
    Line line2 = new Line(0, 3 * Simplicity.getDpmm(), 3 * Simplicity.getDpmm(), 0);

    /**
     * Constructor
     */
    public SCheckBox() {
        this.line1.setStroke(Color.rgb(153, 153, 153));
        this.line1.setStrokeWidth(0.5 * Simplicity.getDpmm());
        this.line2.setStroke(Color.rgb(153, 153, 153));
        this.line2.setStrokeWidth(0.5 * Simplicity.getDpmm());

        this.setAlignment(Pos.CENTER_LEFT);

        this.getChildren().addAll(this.checkBoxStackPane, this.checkBoxSLabel);
        setMargin(checkBoxStackPane, new Insets(0, 1 * Simplicity.getDpmm(), 0, 0));
        setOnMouseClicked(this::handleMouseClicked);
        this.checkBoxStackPane.setMinSize(6 * Simplicity.getDpmm(), 6 * Simplicity.getDpmm());
        this.checkBoxStackPane.setMaxSize(6 * Simplicity.getDpmm(), 6 * Simplicity.getDpmm());
        this.checkBoxStackPane.setStyle("-fx-background-radius:0;-fx-background-color: rgb(255,255,255);-fx-border-color:rgb(" + Simplicity.getDefaultWindowColourSkin().getRGBStringForCSS(ColourVariant.DARK) + ");-fx-border-width: " + Simplicity.getDpmm() / 2 + ";");
    }

    /**
     *
     * @param text
     */
    public SCheckBox(String text) {
        this();
        this.setCheckBoxText(text);
    }

    /**
     *
     * @param text
     */
    public void setCheckBoxText(String text) {
        this.checkBoxSLabel.setText(text);
    }

    /**
     *
     * @return
     */
    public String getCheckBoxText() {
        return this.checkBoxSLabel.getText();
    }

    private void handleMouseClicked(MouseEvent event) {//NOSONAR
        this.setSelected(!this.isSelected());
    }

    public void setSelected(boolean value) {
        this.selected = value;
        this.checkBoxStackPane.getChildren().clear();
        if (value) {
            this.checkBoxStackPane.getChildren().addAll(this.line1, this.line2);
        }
    }

    /**
     *
     * @return
     */
    public boolean isSelected() {
        return this.selected;
    }

    /**
     *
     * @param value
     */
    public void turnDisabled(boolean value) {
        this.disabled = value;
        if (value) {
            this.checkBoxStackPane.setStyle("-fx-background-color: rgb(192,192,192);-fx-border-color:rgb(153,153,153);-fx-border-width: " + Simplicity.getDpmm() / 2 + ";");
            setOnMouseClicked(null);
        } else {
            this.checkBoxStackPane.setStyle("-fx-background-color: rgb(255,255,255);-fx-border-color:rgb(" + Simplicity.getDefaultWindowColourSkin().getRGBStringForCSS(ColourVariant.DARK) + ");-fx-border-width: " + Simplicity.getDpmm() / 2 + ";");
            setOnMouseClicked(this::handleMouseClicked);
        }
    }

    /**
     *
     * @return
     */
    public boolean getDisabled() {
        return this.disabled;
    }

}
