/*
 *License Header
 
 * 
 */
package okay.simplicity.window.titlebuttons;

import javafx.geometry.Pos;
import javafx.scene.paint.Color;
import javafx.scene.shape.Polyline;
import okay.simplicity.Simplicity;
import okay.simplicity.window.Window;
import okay.simplicity.window.WindowColourSkin;

/**
 * Represents maximize/restore button.
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class MaximizeRestoreButton extends WindowTitleButton {

    private Polyline restorePolyline = new Polyline(
            1.5 * Simplicity.getDpmm(), 1 * Simplicity.getDpmm(),
            1.5 * Simplicity.getDpmm(), 0,
            5.5 * Simplicity.getDpmm(), 0,
            5.5 * Simplicity.getDpmm(), 2.0 * Simplicity.getDpmm(),
            4 * Simplicity.getDpmm(), 2.0 * Simplicity.getDpmm(),
            4 * Simplicity.getDpmm(), 3.0 * Simplicity.getDpmm(),
            0 * Simplicity.getDpmm(), 3.0 * Simplicity.getDpmm(),
            0 * Simplicity.getDpmm(), 1 * Simplicity.getDpmm(),
            4 * Simplicity.getDpmm(), 1 * Simplicity.getDpmm(),
            4 * Simplicity.getDpmm(), 2.0 * Simplicity.getDpmm()
    );
    private Polyline maximizePolyline = new Polyline(
            0.25 * Simplicity.getDpmm(), 0.25 * Simplicity.getDpmm(),
            5.25 * Simplicity.getDpmm(), 0.25 * Simplicity.getDpmm(),
            5.25 * Simplicity.getDpmm(), 3 * Simplicity.getDpmm(),
            0.25 * Simplicity.getDpmm(), 3 * Simplicity.getDpmm(),
            0.25 * Simplicity.getDpmm(), 0.25 * Simplicity.getDpmm());

    /**
     *
     * @param windowColourSkin
     * @param window
     */
    public MaximizeRestoreButton(WindowColourSkin windowColourSkin, Window window) {
        super(windowColourSkin);
        setOnMouseClicked(window::handleMaximizeRestoreButtonAction);
        restorePolyline.setStroke(this.windowColourSkin.getColourForInactiveWindowTitleButton());
        restorePolyline.setStrokeWidth(0.5 * Simplicity.getDpmm());
        restorePolyline.setFill(Color.TRANSPARENT);

        maximizePolyline.setStroke(this.windowColourSkin.getColourForInactiveWindowTitleButton());
        maximizePolyline.setStrokeWidth(0.5 * Simplicity.getDpmm());
        maximizePolyline.setFill(Color.TRANSPARENT);

        this.setAlignment(Pos.CENTER);
        this.setMaximizeRestoreButtonMaximized();

    }

    /**
     * Changes button's look to restored.
     */
    public void setMaximizeRestoreButtonRestored() {
        getChildren().clear();
        getChildren().add(restorePolyline);

    }

    /**
     ** Changes button's look to maximized.
     */
    public void setMaximizeRestoreButtonMaximized() {
        getChildren().clear();
        getChildren().add(maximizePolyline);

    }

    @Override
    public void makeActive() {
        restorePolyline.setStroke(this.windowColourSkin.getColourForActiveWindowTitleButton());
        maximizePolyline.setStroke(this.windowColourSkin.getColourForActiveWindowTitleButton());

    }

    @Override
    public void makeInactive() {
        restorePolyline.setStroke(this.windowColourSkin.getColourForInactiveWindowTitleButton());
        maximizePolyline.setStroke(this.windowColourSkin.getColourForInactiveWindowTitleButton());
    }
}
