package okay.simplicity.window.titlebuttons;

import javafx.scene.input.MouseEvent;
import javafx.scene.layout.StackPane;
import javafx.scene.shape.Line;
import okay.simplicity.Simplicity;
import okay.simplicity.window.ColourVariant;
import okay.simplicity.window.WindowColourSkin;

/**
 * Represents window title button.
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public abstract class WindowTitleButton extends StackPane {

    private static final int BUTTONWIDTH = 7;

    protected WindowColourSkin windowColourSkin;

    /**
     *
     * @param windowColourSkin
     */
    public WindowTitleButton(WindowColourSkin windowColourSkin) {
        this.windowColourSkin = windowColourSkin;
        this.setStyle("-fx-background-color: rgb(" + windowColourSkin.getRGBStringForCSS(ColourVariant.DARK) + ");");
        setMinWidth(BUTTONWIDTH * Simplicity.getDpmm());
        setMaxWidth(BUTTONWIDTH * Simplicity.getDpmm());

        setOnMouseEntered(this::handleMouseEnteredAction);
        setOnMouseExited(this::handleMMouseExitedAction);
    }

    /**
     * Makes the background of the button active.
     */
    public void makeActiveBackground() {
        this.setStyle("-fx-background-color: rgb(212, 212, 212);");
    }

    /**
     *
     * @param line
     */
    public void makeLine(Line line) {
        line.setStroke(this.windowColourSkin.getColourForInactiveWindowTitleButton());
        line.setStrokeWidth(0.5 * Simplicity.getDpmm());
    }

    private void handleMouseEnteredAction(MouseEvent event) {//NOSONAR
        makeActive();
    }

    private void handleMMouseExitedAction(MouseEvent event) {//NOSONAR
        makeInactive();
    }

    /**
     * Makes button to look active.
     */
    public abstract void makeActive();

    /**
     * Makes button to look inactive.
     */
    public abstract void makeInactive();

}
