/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package okay.database;

import java.io.File;
import okay.OkayRuntimeException;
import okay.pseudorandom.PseudoRandomGenerator;
import org.junit.AfterClass;
import org.junit.Test;
import static org.junit.Assert.*;
import org.junit.BeforeClass;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class DatabaseTest {

    private static final File directory;
    private static final String directoryName;

    static {
        PseudoRandomGenerator pseudoRandomGenerator = PseudoRandomGenerator.getInstance();
        int randomNumber = pseudoRandomGenerator.getInt(0, 999999);
        directoryName = "./temp" + randomNumber + "/";
        directory = new File(directoryName);
    }

    static public boolean deleteDirectory(File path) {
        if (path.exists()) {
            File[] files = path.listFiles();
            for (int i = 0; i < files.length; i++) {
                if (files[i].isDirectory()) {
                    deleteDirectory(files[i]);
                } else {
                    files[i].delete();
                }
            }
        }
        return (path.delete());
    }

    public DatabaseTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
        directory.mkdir();
    }

    private String getNextDatabaseName() {
        return Integer.toString(PseudoRandomGenerator.getInstance().getInt(0, 1000000));
    }

    /**
     * Test of createDatabase method, of class Database.
     */
    @Test
    public void testCreateDatabase() {
        //arrange
        String databaseName = directoryName + getNextDatabaseName();
        //act
        Database.createDatabase(databaseName);
        //assert
        assertTrue(new File(databaseName + ".sqlite").exists());
    }

    /**
     * Test of createDatabase method, of class Database.
     */
    @Test
    public void testCreateDatabase2() {
        //arrange
        boolean isExceptionThrown = false;
        String databaseName = directoryName + getNextDatabaseName();
        //act
        Database.createDatabase(databaseName);
        try {
            Database.createDatabase(databaseName);
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of dropDatabase method, of class Database.
     */
    @Test
    public void testDropDatabase() {
        //arrange
        String databaseName = directoryName + getNextDatabaseName();
        Database.createDatabase(databaseName);
        //act
        Database.dropDatabase(databaseName);
        //assert
        assertFalse(Database.existsDatabase(databaseName));
    }

    /**
     * Test of dropDatabase method, of class Database.
     */
    @Test
    public void testDropDatabase2() {
        //arrange
        boolean isExceptionThrown = false;
        String databaseName = directoryName + getNextDatabaseName();
        //act
        try {
            Database.dropDatabase(databaseName);
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of existsDatabaseWithThisName method, of class Database.
     */
    @Test
    public void testExistsDatabaseWithThisName() {
        //arrange
        String databaseName = directoryName + getNextDatabaseName();
        //act
        Database.createDatabase(databaseName);
        //assert
        assertTrue(Database.existsDatabase(databaseName));
    }

    /**
     * Test of existsDatabaseWithThisName method, of class Database.
     */
    @Test
    public void testExistsDatabaseWithThisName2() {
        //arrange
        String databaseName = directoryName + getNextDatabaseName();
        //act
        //assert
        assertFalse(Database.existsDatabase(databaseName));
    }

    /**
     * Test of createDatabaseConnection method, of class Database.
     */
    @Test
    public void testCreateDatabaseConnection() {
        //arrange
        boolean isExceptionThrown = false;
        String databaseName = directoryName + getNextDatabaseName();
        //act
        Database.createDatabase(databaseName);
        try {
            Database.createDatabaseConnection(databaseName);
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (isExceptionThrown) {
            fail("There should be thrown no OkayRuntimeException");
        }
    }

    /**
     * Test of createDatabaseConnection method, of class Database.
     */
    @Test
    public void testCreateDatabaseConnection2() {
        //arrange
        boolean isExceptionThrown = false;
        String databaseName = directoryName + getNextDatabaseName();
        //act
        try {
            Database.createDatabaseConnection(databaseName);
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    @AfterClass
    public static void tearDownClass() {
        deleteDirectory(directory);
    }
}
