package okay.datetime;

import okay.OkayRuntimeException;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class LocalTimeTest {

    /**
     *
     */
    public LocalTimeTest() {
    }

    /**
     * Test of constructor of class LocalTime.
     */
    @Test
    public void testDateTime_ThereShouldBeThrownNoOkayRuntimeException() {
        //arrange
        boolean isExceptionThrown = false;
        //act
        try {
            LocalTime localTime = new LocalTime(21, 45, 6, 32);
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (isExceptionThrown) {
            fail("There should be thrown no OkayRuntimeException");
        }
    }

    /**
     * Test of constructor of class LocalTime.
     */
    @Test
    public void testDateTime_HourIsMinusOne_HourIsOutOfRange_ThereShouldBeThrownOkayRuntimeException() {
        //arrange
        boolean isExceptionThrown = false;
        //act
        try {
            LocalTime localTime = new LocalTime(-1, 45, 6, 32);
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of constructor of class LocalTime.
     */
    @Test
    public void testDateTime_HourIsTwentyFour_HourIsOutOfRange_ThereShouldBeThrownOkayRuntimeException() {
        //arrange
        boolean isExceptionThrown = false;
        //act
        try {
            LocalTime localTime = new LocalTime(24, 45, 6, 32);
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of constructor of class LocalTime.
     */
    @Test
    public void testDateTime_MinuteIsMinusOne_MinuteIsOutOfRange_ThereShouldBeThrownOkayRuntimeException() {

        //arrange
        boolean isExceptionThrown = false;
        //act
        try {
            LocalTime localTime = new LocalTime(21, -1, 6, 32);
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of constructor of class LocalTime.
     */
    @Test
    public void testDateTime_MinuteIsSixty_MinuteIsOutOfRange_ThereShouldBeThrownOkayRuntimeException() {
        //arrange
        boolean isExceptionThrown = false;
        //act
        try {
            LocalTime localTime = new LocalTime(21, 60, 6, 32);
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of constructor of class LocalTime.
     */
    @Test
    public void testDateTime_SecondIsMinusOne_SecondIsOutOfRange_ThereShouldBeThrownOkayRuntimeException() {
        //arrange
        boolean isExceptionThrown = false;
        //act
        try {
            LocalTime localTime = new LocalTime(21, 45, -1, 32);
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of constructor of class LocalTime.
     */
    @Test
    public void testDateTime_SecondIsSixty_SecondIsOutOfRange_ThereShouldBeThrownOkayRuntimeException() {
        //arrange
        boolean isExceptionThrown = false;
        //act
        try {
            LocalTime localTime = new LocalTime(21, 45, 60, 32);
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of constructor of class LocalTime.
     */
    @Test
    public void testDateTime_MillisecondIsMinusOne_MillisecondIsOutOfRange_ThereShouldBeThrownOkayRuntimeException() {
        //arrange
        boolean isExceptionThrown = false;
        //act
        try {
            LocalTime localTime = new LocalTime(21, 45, 6, -1);
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of constructor of class LocalTime.
     */
    @Test
    public void testDateTime_MillisecondIsOneThousand_MillisecondIsOutOfRange_ThereShouldBeThrownOkayRuntimeException() {
        //arrange
        boolean isExceptionThrown = false;
        //act
        try {
            LocalTime localTime = new LocalTime(21, 45, 6, 1000);
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of getHour method, of class LocalTime.
     */
    @Test
    public void testGetHour() {
        //arrange
        int expectedHour = 9;
        int returnedHour;
        LocalTime localTime;
        //act
        localTime = new LocalTime(9, 23, 14, 354);
        returnedHour = localTime.getHour();
        //assert
        assertEquals(expectedHour, returnedHour);
    }

    /**
     * Test of getMinute method, of class LocalTime.
     */
    @Test
    public void testGetMinute() {
        //arrange
        int expectedMinute = 23;
        int returnedMinute;
        LocalTime localTime;
        //act
        localTime = new LocalTime(9, 23, 14, 354);
        returnedMinute = localTime.getMinute();
        //assert
        assertEquals(expectedMinute, returnedMinute);
    }

    /**
     * Test of getSecond method, of class LocalTime.
     */
    @Test
    public void testGetSecond() {
        //arrange
        int expectedSecond = 14;
        int returnedSecond;
        LocalTime localTime;
        //act
        localTime = new LocalTime(9, 23, 14, 354);
        returnedSecond = localTime.getSecond();
        //assert
        assertEquals(expectedSecond, returnedSecond);
    }

    /**
     * Test of getMillisecond method, of class LocalTime.
     */
    @Test
    public void testGetMillisecond() {
        //arrange
        int expectedMillisecond = 354;
        int returnedMillisecond;
        LocalTime localTime;
        //act
        localTime = new LocalTime(9, 23, 14, 354);
        returnedMillisecond = localTime.getMillisecond();
        //assert
        assertEquals(expectedMillisecond, returnedMillisecond);
    }

    /**
     * Test of toString method, of class DateTime.
     */
    @Test
    public void testToString() {
        //arrange
        String expectedResult = "09:23:14:354";
        String result;
        LocalTime localTime;
        //act
        localTime = new LocalTime(9, 23, 14, 354);
        result = localTime.toString();
        //assert
        assertEquals(expectedResult, result);
    }
}
