package okay.json;

import okay.OkayRuntimeException;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class JsonValueTest {

    /**
     *
     */
    public JsonValueTest() {
    }

    /**
     * Test of getJsonValueType method, of class JsonValue.
     */
    @Test
    public void testGetJsonValueType() {
        //arrange
        JsonValue jsonValue;
        JsonValueType expectedValue;
        JsonValueType returnedValue = JsonValueType.STRING;
        JsonString jsonString;
        //act
        jsonString = new JsonString("hello");
        jsonValue = new JsonValue(jsonString);
        expectedValue = jsonValue.getJsonValueType();
        //assert
        assertEquals(expectedValue, returnedValue);
    }

    /**
     * Test of isNull method, of class JsonValue.
     */
    @Test
    public void testIsNull() {
        //arrange
        JsonValue jsonValue;
        JsonValueType expectedValue;
        JsonValueType returnedValue = JsonValueType.NULL;
        //act
        jsonValue = new JsonValue();
        expectedValue = jsonValue.getJsonValueType();
        //assert
        assertEquals(expectedValue, returnedValue);
    }

    /**
     * Test of isObject method, of class JsonValue.
     */
    @Test
    public void testIsObject() {
        //arrange
        JsonValue jsonValue;
        JsonValueType expectedValue;
        JsonValueType returnedValue = JsonValueType.OBJECT;
        JsonObject jsonObject;
        //act
        jsonObject = new JsonObject();
        jsonValue = new JsonValue(jsonObject);
        expectedValue = jsonValue.getJsonValueType();
        //assert
        assertEquals(expectedValue, returnedValue);
    }

    /**
     * Test of isArray method, of class JsonValue.
     */
    @Test
    public void testIsArray() {
//arrange
        JsonValue jsonValue;
        JsonValueType expectedValue;
        JsonValueType returnedValue = JsonValueType.ARRAY;
        JsonArray jsonArray;
        //act
        jsonArray = new JsonArray();
        jsonValue = new JsonValue(jsonArray);
        expectedValue = jsonValue.getJsonValueType();
        //assert
        assertEquals(expectedValue, returnedValue);
    }

    /**
     * Test of isBoolean method, of class JsonValue.
     */
    @Test
    public void testIsBoolean() {
//arrange
        JsonValue jsonValue;
        JsonValueType expectedValue;
        JsonValueType returnedValue = JsonValueType.BOOLEAN;
        JsonBoolean jsonBoolean;
        //act
        jsonBoolean = new JsonBoolean(true);
        jsonValue = new JsonValue(jsonBoolean);
        expectedValue = jsonValue.getJsonValueType();
        //assert
        assertEquals(expectedValue, returnedValue);
    }

    /**
     * Test of isString method, of class JsonValue.
     */
    @Test
    public void testIsString() {
//arrange
        JsonValue jsonValue;
        JsonValueType expectedValue;
        JsonValueType returnedValue = JsonValueType.STRING;
        JsonString jsonString;
        //act
        jsonString = new JsonString("hello");
        jsonValue = new JsonValue(jsonString);
        expectedValue = jsonValue.getJsonValueType();
        //assert
        assertEquals(expectedValue, returnedValue);
    }

    /**
     * Test of isChar method, of class JsonValue.
     */
    @Test
    public void testIsChar() {
//arrange
        JsonValue jsonValue;
        JsonValueType expectedValue;
        JsonValueType returnedValue = JsonValueType.CHAR;
        JsonChar jsonChar;
        //act
        jsonChar = new JsonChar('h');
        jsonValue = new JsonValue(jsonChar);
        expectedValue = jsonValue.getJsonValueType();
        //assert
        assertEquals(expectedValue, returnedValue);
    }

    /**
     * Test of isInt method, of class JsonValue.
     */
    @Test
    public void testIsInt() {
//arrange
        JsonValue jsonValue;
        JsonValueType expectedValue;
        JsonValueType returnedValue = JsonValueType.INT;
        JsonInt jsonInt;
        //act
        jsonInt = new JsonInt(549);
        jsonValue = new JsonValue(jsonInt);
        expectedValue = jsonValue.getJsonValueType();
        //assert
        assertEquals(expectedValue, returnedValue);
    }

    /**
     * Test of isLong method, of class JsonValue.
     */
    @Test
    public void testIsLong() {
//arrange
        JsonValue jsonValue;
        JsonValueType expectedValue;
        JsonValueType returnedValue = JsonValueType.LONG;
        JsonLong jsonLong;
        //act
        jsonLong = new JsonLong(4l);
        jsonValue = new JsonValue(jsonLong);
        expectedValue = jsonValue.getJsonValueType();
        //assert
        assertEquals(expectedValue, returnedValue);
    }

    /**
     * Test of isFloat method, of class JsonValue.
     */
    @Test
    public void testIsFloat() {
//arrange
        JsonValue jsonValue;
        JsonValueType expectedValue;
        JsonValueType returnedValue = JsonValueType.FLOAT;
        JsonFloat jsonFloat;
        //act
        jsonFloat = new JsonFloat(8.6f);
        jsonValue = new JsonValue(jsonFloat);
        expectedValue = jsonValue.getJsonValueType();
        //assert
        assertEquals(expectedValue, returnedValue);
    }

    /**
     * Test of isDouble method, of class JsonValue.
     */
    @Test
    public void testIsDouble() {
//arrange
        JsonValue jsonValue;
        JsonValueType expectedValue;
        JsonValueType returnedValue = JsonValueType.DOUBLE;
        JsonDouble jsonDouble;
        //act
        jsonDouble = new JsonDouble(8.6d);
        jsonValue = new JsonValue(jsonDouble);
        expectedValue = jsonValue.getJsonValueType();
        //assert
        assertEquals(expectedValue, returnedValue);
    }

    /**
     * Test of getJsonObject method, of class JsonValue.
     */
    @Test
    public void testGetJsonObject() {
        //arrange
        boolean isExceptionThrown = false;
        JsonString jsonString = new JsonString("hello");
        JsonValue jsonValue = new JsonValue(jsonString);
        //act
        try {
            jsonValue.getJsonObject();
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of getJsonArray method, of class JsonValue.
     */
    @Test
    public void testGetJsonArray() {

        //arrange
        boolean isExceptionThrown = false;
        JsonString jsonString = new JsonString("hello");
        JsonValue jsonValue = new JsonValue(jsonString);
        //act
        try {
            jsonValue.getJsonArray();
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of getJsonBoolean method, of class JsonValue.
     */
    @Test
    public void testGetJsonBoolean() {

        //arrange
        boolean isExceptionThrown = false;
        JsonString jsonString = new JsonString("hello");
        JsonValue jsonValue = new JsonValue(jsonString);
        //act
        try {
            jsonValue.getJsonBoolean();
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of getJsonString method, of class JsonValue.
     */
    @Test
    public void testGetJsonString() {

        //arrange
        boolean isExceptionThrown = false;
        JsonString jsonString = new JsonString("hello");
        JsonValue jsonValue = new JsonValue(jsonString);
        //act
        try {
            jsonValue.getJsonString();
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (isExceptionThrown) {
            fail("There should be thrown no OkayRuntimeException");
        }
    }

    /**
     * Test of getJsonChar method, of class JsonValue.
     */
    @Test
    public void testGetJsonChar() {

        //arrange
        boolean isExceptionThrown = false;
        JsonString jsonString = new JsonString("hello");
        JsonValue jsonValue = new JsonValue(jsonString);
        //act
        try {
            jsonValue.getJsonChar();
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of getJsonInt method, of class JsonValue.
     */
    @Test
    public void testGetJsonInt() {

        //arrange
        boolean isExceptionThrown = false;
        JsonString jsonString = new JsonString("hello");
        JsonValue jsonValue = new JsonValue(jsonString);
        //act
        try {
            jsonValue.getJsonInt();
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of getJsonLong method, of class JsonValue.
     */
    @Test
    public void testGetJsonLong() {

        //arrange
        boolean isExceptionThrown = false;
        JsonString jsonString = new JsonString("hello");
        JsonValue jsonValue = new JsonValue(jsonString);
        //act
        try {
            jsonValue.getJsonLong();
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of getJsonFloat method, of class JsonValue.
     */
    @Test
    public void testGetJsonFloat() {

        //arrange
        boolean isExceptionThrown = false;
        JsonString jsonString = new JsonString("hello");
        JsonValue jsonValue = new JsonValue(jsonString);
        //act
        try {
            jsonValue.getJsonFloat();
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

    /**
     * Test of getJsonDouble method, of class JsonValue.
     */
    @Test
    public void testGetJsonDouble() {

        //arrange
        boolean isExceptionThrown = false;
        JsonString jsonString = new JsonString("hello");
        JsonValue jsonValue = new JsonValue(jsonString);
        //act
        try {
            jsonValue.getJsonDouble();
        } catch (OkayRuntimeException e) {
            isExceptionThrown = true;
        }
        //assert
        if (!isExceptionThrown) {
            fail("There should be thrown OkayRuntimeException");
        }
    }

}
